package com.hello2morrow.sonargraph.languageprovider.java.command.system;

import static org.junit.Assert.assertEquals
import static org.junit.Assert.assertNotNull
import static org.junit.Assert.assertTrue

import org.junit.Rule
import org.junit.Test
import org.junit.rules.TemporaryFolder

import com.hello2morrow.foundation.utilities.OperationResultWithOutcome
import com.hello2morrow.sonargraph.core.command.system.CreateReportCommand
import com.hello2morrow.sonargraph.core.controller.system.SoftwareSystemController.OptionalExtensionId
import com.hello2morrow.sonargraph.core.model.element.Issue
import com.hello2morrow.sonargraph.core.model.element.IssueCategory
import com.hello2morrow.sonargraph.core.model.metrics.CoreMetricLevel
import com.hello2morrow.sonargraph.core.model.path.CoreFileType
import com.hello2morrow.sonargraph.core.model.report.IReport
import com.hello2morrow.sonargraph.core.model.resolution.IssueFilter
import com.hello2morrow.sonargraph.core.model.resolution.ResolutionMode
import com.hello2morrow.sonargraph.core.model.system.ISoftwareSystemProvider
import com.hello2morrow.sonargraph.core.model.system.SoftwareSystem
import com.hello2morrow.sonargraph.core.model.system.SoftwareSystemState
import com.hello2morrow.sonargraph.core.model.system.VirtualModel
import com.hello2morrow.sonargraph.languageprovider.java.foundation.common.JavaTestFixture

import de.schlichtherle.truezip.file.TFile
import de.schlichtherle.truezip.file.TFileInputStream

public class CreateXmlReportWithClassFileIssuesTest extends AbstractJavaCommandTest
{
    private static final String SYSTEM_PATH = JavaTestFixture.TINY_TEST_PROJECT_XML_REPORT_CLASSFILE_ISSUES;
    private static final String REPORT_FILE_NAME = "Test";

    private OperationResultWithOutcome<IReport> m_result;

    @Rule
    public final TemporaryFolder m_reportDirectory = new TemporaryFolder();

    public CreateXmlReportWithClassFileIssuesTest()
    {
        super(SYSTEM_PATH);
    }

    @Test
    public void checkForSourceAttributeInClassFileElement()
    {
        openAndClearAndRefreshSystem(new TFile(SYSTEM_PATH), EnumSet.of(ISoftwareSystemProvider.OpenOption.SWITCH_TO_DEFAULT_MODEL),
                EnumSet.allOf(OptionalExtensionId.class));
        final SoftwareSystem softwareSystem = getController().getSoftwareSystem();
        assertEquals("Wrong software system state", SoftwareSystemState.MODEL_LOADED, softwareSystem.getState());

        final VirtualModel currentModel = softwareSystem.getCurrentModel();

        final IssueFilter filter = new IssueFilter();
        filter.add(ResolutionMode.NONE);
        filter.add(IssueCategory.SCRIPT_BASED);
        final List<Issue> issues = currentModel.getIssueList(filter).getIssues();
        assertEquals("Wrong number of issues", 1, issues.size());

        final TFile reportDirectory = new TFile(m_reportDirectory.getRoot());
        final CreateReportCommand reportCommand = new CreateReportCommand(getController(), new CreateReportCommand.IReportInteraction()
                {
                    @Override
                    public void handleResult(final OperationResultWithOutcome<IReport> result)
                    {
                        m_result = result;
                    }

                    @Override
                    public boolean collect(final CreateReportCommand.ReportInteractionData data)
                    {
                        data.setReportFormats(EnumSet.of(IReport.Format.XML));
                        data.setReportDirectory(reportDirectory);
                        data.setReportFileName(REPORT_FILE_NAME);
                        data.setMetricLevels(Arrays.asList(CoreMetricLevel.SYSTEM));
                        return true;
                    }
                });
        reportCommand.runSynchronously();

        assertNotNull("No result available", m_result);
        assertTrue(m_result.toString(), m_result.isSuccess());

        final TFile reportFile = new TFile(reportDirectory, REPORT_FILE_NAME + CoreFileType.XML_EXTENSION);
        assertTrue("Report file does not exist", reportFile.exists());

        def reportXml = new XmlSlurper().parse(new TFileInputStream(reportFile));
        assertEquals("Wrong root element name", "report", reportXml.name());
        assertEquals("Wrong number of moduleElement sections", 1, reportXml.moduleElements.children().size());
        assertEquals("Wrong number of moduleElement children", 1, reportXml.moduleElements[0].children().size());

        def element = reportXml.moduleElements[0].element[0];
        assertEquals("Wrong attribute value", "./com/h2m/tiny/Simple.class", element.'@presentationName'.toString());
        def sourceRef = element.'@source'.toString();
        assertTrue("Source reference missing", sourceRef != null && sourceRef.startsWith("_"));
    }
}
